<?php
/**
 * @file
 *   Commerce Xero Admin page callabcks.
 */


/**
 * Commerce Xero Account Form.
 *
 * @param $form
 * @param &$form_state
 * @param $bank
 *   The Xero Bank Account object if it exists.
 * @return array
 */
function commerce_xero_account_form($form, &$form_state, $bank = NULL) {

  $bank_options = commerce_xero_bank_account_options();
  $rev_options = commerce_xero_revenue_account_options();
  $method_options = commerce_payment_method_options_list();

  if (empty($bank_options)) {
    drupal_set_message(t('Could not find any bank accounts associated with your Xero company.'), 'error');
    return;
  }
  elseif (empty($rev_options)) {
    drupal_set_message(t('Could not find any revenue accounts associated with your Xero company.'), 'error');
    return;
  }
  elseif (empty($method_options)) {
    drupal_set_message(t('Could not find any Drupal Commerce payment methods.'), 'error');
    return;
  }

  $form['payment_method'] = array(
    '#type' => 'select',
    '#title' => t('Commerce payment method'),
    '#description' => t('Choose a payment method. You may only associate a payment method once.'),
    '#options' => $method_options,
    '#default_value' => isset($bank) ? $bank->payment_method : '',
    '#required' => TRUE,
    '#disabled' => isset($bank) ? TRUE : FALSE,
  );

  $form['bank_account'] = array(
    '#type' => 'select',
    '#title' => t('Xero bank account'),
    '#description' => t('Choose the bank account to associate to a payment method.'),
    '#options' => $bank_options,
    '#default_value' => isset($bank) ? $bank->bank_account : '',
    '#required' => TRUE,
  );

  $form['revenue_code'] = array(
    '#type' => 'select',
    '#title' => t('Xero account code'),
    '#description' => t('Choose the account code to post transactions to.'),
    '#options' => $rev_options,
    '#default_value' => isset($bank) ? $bank->revenue_code : '',
    '#required' => TRUE,
  );

  $form['transaction_type'] = array(
    '#type' => 'select',
    '#title' => t('Xero transaction type'),
    '#description' => t('Choose the type of transaction to make; either a bank transaction or invoice payments.'),
    '#options' => array('BankTransaction' => t('Bank transaction'), 'Payment' => t('Invoice payment')),
    '#default_value' => isset($bank) ? $bank->transaction_type : '',
  );

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 40,
  );

  if ($bank) {
    $form_state['account'] = $bank;

    $form['actions']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#submit' => array('commerce_xero_account_form_delete'),
      '#weight' => 42,
    );
  }

  return $form;
}

/**
 * Commerce Xero Account Form Validate
 */
function commerce_xero_account_form_validate($form, &$form_state) {
  if (!isset($form_state['account'])) {
    $query = db_select('commerce_xero_bankaccount', 'xba');
    $query->condition('payment_method', $form_state['values']['payment_method']);
    $num = $query->countQuery()->execute()->fetchField();

    if ($num) {
      form_set_error('payment_method', t('This payment method is already associated with a bank account.'));
    }
  }
}

/**
 * Commerce Xero Account Form Delete 
 */
function commerce_xero_account_form_delete($form, &$form_state) {
  $form_state['redirect'] = 'admin/commerce/config/xero/account/' . $form_state['account']->id . '/delete';
}

/**
 * Commerce Xero Account Form Submit.
 */
function commerce_xero_account_form_submit($form, &$form_state) {

  if (isset($form_state['account'])) {
    $bank = $form_state['account'];
  }
  else {
    $bank = (object) array(
      'payment_method' => '',
      'revenue_code' => '',
      'bank_account' => '',
      'transaction_type' => '',
    );
  }

  $bank->payment_method = $form_state['values']['payment_method'];
  $bank->revenue_code = $form_state['values']['revenue_code'];
  $bank->bank_account = $form_state['values']['bank_account'];
  $bank->transaction_type = $form_state['values']['transaction_type'];

  if (isset($bank->id)) {
    $ret = drupal_write_record('commerce_xero_bankaccount', $bank, array('id'));
  }
  else {
    $ret = drupal_write_record('commerce_xero_bankaccount', $bank);

    // Create field instance.
    $instance_info = array(
      'field_name' => 'field_xero_transaction',
      'label' => t('Xero transaction'),
      'entity_type' => 'commerce_payment_transaction',
      'bundle' => $bank->payment_method,
      'widget' => array(
        'type' => 'xero_textfield',
        'settings' => array(
          'xero_type' => array('BankTransaction', 'Payment'),
        ),  
      ),  
      'display' => array(
        'default' => array(
          'type' => 'xero_reference',
          'label' => 'inline',
          'settings' => array(
            'display' => array('type', 'guid'),
          ),  
        ),
      ),
    );
    $instance = field_create_instance($instance_info);
  }

  if ($ret) {
    drupal_set_message(t('Xero bank account %name successfully saved.', array('%name' => $bank->bank_account)));
    $form_state['redirect'] = 'admin/commerce/config/xero/account/' . $bank->id;
  }
  else {
    drupal_set_message(t('Failed to create Xero bank account.'), 'error');
  }

}

/**
 * Commerce Xero Bank Account Delete Form.
 */
function commerce_xero_account_delete_form($form, &$form_state, $bank) {

  $form['help'] = array(
    '#type' => 'container',
    '#attributes' => array(
      'class' => array('messages', 'warning'),
    ),
    'message' => array(
      '#markup' => t('Are you sure you want to permanently delete %name? This will not remove any created transactions.', array('%name' => $bank->bank_account)),
    ),
  );

  $form_state['account'] = $bank;

  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Permanently remove'),
  );
  $form['actions']['cancel'] = array(
    '#markup' => l(t('Cancel'), 'admin/commerce/config/xero'),
  );

  return $form;
}

/**
 * Commerce Xero Bank Account Delete Form submit callback.
 */
function commerce_xero_account_delete_form_submit($form, &$form_state) {

  $instance = array(
    'field_name' => 'field_xero_transaction',
    'entity_type' => 'commerce_payment_transaction',
    'bundle' => $form_state['account']->bank_account,
  );
  field_delete_instance($instance);

  $num = db_delete('commerce_xero_bankaccount')->condition('id', $form_state['account']->id)->execute();

  if ($num) {
    drupal_set_message(t('Successfully deleted %name.', array('%name' => $form_state['account']->bank_account)));
    $form_state['redirect'] = 'admin/commerce/config/xero';
  }
  else {
    drupal_set_message(t('Unable to delete bank account!'), 'error');
  }
}
